package com.cneeds.notouch;

import android.content.ContentValues;
import android.content.Context;
import android.database.ContentObserver;
import android.database.Cursor;
import android.net.Uri;
import android.os.Handler;
import android.os.Looper;
import android.os.Message;
import android.os.Process;
import android.view.KeyEvent;

import com.cneeds.notouch.bean.ActivityOperationBean;
import com.cneeds.notouch.bean.CoordinateBean;
import com.cneeds.notouch.bean.OperationBean;
import com.cneeds.notouch.operation.NoTouchOperation;
import com.cneeds.notouch.operation.NoTouchOperationFactory;


import java.util.HashMap;

import static com.cneeds.notouch.NoTouchConstant.ColumnName.COLUMN_FOCUS_KEY;
import static com.cneeds.notouch.NoTouchConstant.ColumnName.COLUMN_FOCUS_NAME;
import static com.cneeds.notouch.NoTouchConstant.ColumnName.COLUMN_FOCUS_NEED;
import static com.cneeds.notouch.NoTouchConstant.ColumnName.COLUMN_FOCUS_STATE;
import static com.cneeds.notouch.NoTouchConstant.ColumnName.COLUMN_FOCUS_TEMP;
import static com.cneeds.notouch.NoTouchConstant.Focus.APP_FOCUS;
import static com.cneeds.notouch.NoTouchConstant.Focus.SYSTEM_BAR_FOCUS;
import static com.cneeds.notouch.NoTouchConstant.FocusStatus.RECOVER;
import static com.cneeds.notouch.NoTouchConstant.OperationResult.SUCCESS;


/**
 * 非触控的管理类，主要做一些非触控的逻辑操作
 * @author zhany
 */
public class NoTouchFocusController {

    private Uri CONTENT_URI = null;
    private Uri NEED_FOCUS_URL = null;
    private NoTouchListener mNoTouchListener;
    private NoTouchOperation mNoTouchOperation;
    private OperationHandler mOperationHandler;
    private Context mContext;

    private final int MESSAGE_KEY_DOWN = 1;
    private final int MESSAGE_UPDATE_FOCUS = 2;
    private final int MESSAGE_DELAYED = 600;
    private int mCurrentFocus = SYSTEM_BAR_FOCUS;
    private int mNavigationBarKeyCode;
    private NoTouchActivityOperation mNoTouchActivityOperation;
    private NoTouchSelfListener mNoTouchSelfListener;
    private HashMap<String, NoTouchOperation> mNoTouchOperationCache;
    private int mAppLastKeyCode;
    private boolean mSupportHardKey = false;


    protected NoTouchFocusController(Context context){
        try {
            mContext = context;
            mNoTouchOperationCache = new HashMap<>();
            mOperationHandler = new OperationHandler(Looper.getMainLooper());
            mNoTouchActivityOperation = new NoTouchActivityOperation();
            CONTENT_URI = Uri.parse(NoTouchConstant.FOCUS_STATUS_URL);
            NEED_FOCUS_URL = Uri.parse(NoTouchConstant.FOCUS_NEED_URL);
            mContext.getContentResolver().registerContentObserver(CONTENT_URI, false, mNoTouchFocusObserver);
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    public void registerSelfListener(NoTouchSelfListener selfListener){
        updateCurrentFocus();
        sendCurrentNeedFocus(true);
        mNoTouchSelfListener = selfListener;
        mNoTouchSelfListener.onFocusChange(mCurrentFocus);
        mNoTouchListener = null;
    }

    public void unRegisterSelfListener(NoTouchSelfListener selfListener){
        if(mNoTouchSelfListener != null
                && selfListener != null
                && mNoTouchSelfListener.equals(selfListener)){
            sendCurrentNeedFocus(false);
            mNoTouchSelfListener = null;
        }

    }


    private ContentObserver mNoTouchFocusObserver = new ContentObserver(new Handler()) {
        @Override
        public void onChange(boolean selfChange, Uri uri) {
            updateCurrentFocus();
            if(mNoTouchSelfListener != null){
                mNoTouchSelfListener.onFocusChange(mCurrentFocus);
            }
            if(mCurrentFocus == APP_FOCUS && mNoTouchOperation != null){
                if(mNavigationBarKeyCode == KeyEvent.KEYCODE_SOFT_RIGHT){
                    mNoTouchOperation.resetInitCoordinate();
                }
                mNoTouchOperation.restoreLastCoordinate();
            }else if(mNoTouchListener != null){
                mNoTouchListener.onSelectNone();
            }
        }
    };

    public void registerNoTouchTempListener(String packageName){
        try {
            NoTouchLog.d("registerNoTouchTempListener = " + packageName);
            ContentValues values = new ContentValues();
            values.put(COLUMN_FOCUS_NAME, packageName + ":" + Process.myPid());
            values.put(COLUMN_FOCUS_TEMP, true);
            values.put(COLUMN_FOCUS_NEED, true);
            mContext.getContentResolver().update(NEED_FOCUS_URL, values, null, null);
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    public void unRegisterNoTouchTempListener(String packageName){
        NoTouchLog.d("unRegisterNoTouchTempListener = " + packageName);
        try{
            ContentValues values = new ContentValues();
            values.put(COLUMN_FOCUS_NAME, packageName + ":" + Process.myPid());
            values.put(COLUMN_FOCUS_TEMP, true);
            values.put(COLUMN_FOCUS_NEED, false);
            mContext.getContentResolver().update(NEED_FOCUS_URL, values, null, null);
        }catch (Exception e){
            e.printStackTrace();
        }

    }


    public void registerNoTouchListener(NoTouchListener listener, NoTouchParam param){
        mOperationHandler.removeCallbacksAndMessages(null);
        mNoTouchListener = listener;
        mNoTouchSelfListener = null;
        sendCurrentNeedFocus(true);
        boolean isCache = false;
        if(param.getClassName() != null && param.getFocusStatus() == RECOVER){
            mNoTouchOperation = mNoTouchOperationCache.get(param.getClassName());
            isCache = true;
        }else{
            mNoTouchOperation = null;
        }

        if(mNoTouchOperation == null){
            mNoTouchOperation = NoTouchOperationFactory.createOperation(param);
            mNoTouchOperationCache.put(param.getClassName(), mNoTouchOperation);
        }

        mNoTouchOperation.registerActivityOperationResult(mNoTouchActivityOperation);
        if(param.getNoNavigationBar()){
            sendCurrentHasFocus();
        }

        mNoTouchListener.onSelectNone();

        Message message = mOperationHandler.obtainMessage(MESSAGE_UPDATE_FOCUS);
        message.obj = isCache;
        mOperationHandler.sendMessageDelayed(message, MESSAGE_DELAYED);

        if(mNoTouchOperation == null){
            try {
                throw new NullPointerException("NoTouchFocusController mNoTouchOperation IS NULL");
            } catch (Exception e) {
                e.printStackTrace();
            }
        }
    }


    public void unRegisterNoTouchListener(NoTouchListener listener){
        if(mNoTouchListener != null && listener != null
                && mNoTouchListener.equals(listener)){
            mNoTouchListener = null;
            mOperationHandler.removeCallbacksAndMessages(null);
            sendCurrentNeedFocus(false);
        }
    }


    public void onKeyDown(int keyCode, KeyEvent keyEvent){
        checkSupportHardKey(keyCode);
        mAppLastKeyCode = keyCode;
        Message message = mOperationHandler.obtainMessage();
        message.what = MESSAGE_KEY_DOWN;
        message.arg1 = keyCode;
        message.obj = keyEvent;
        message.sendToTarget();
    }

    private void checkSupportHardKey(int keyCode){
        if(mSupportHardKey){
            return;
        }

        mSupportHardKey = checkUseKeyCode(keyCode);
    }

    public boolean checkUseKeyCode(int keyCode){
        if(keyCode == KeyEvent.KEYCODE_DPAD_LEFT
                || keyCode == KeyEvent.KEYCODE_DPAD_UP
                || keyCode == KeyEvent.KEYCODE_DPAD_RIGHT
                || keyCode == KeyEvent.KEYCODE_DPAD_DOWN
                || keyCode == KeyEvent.KEYCODE_SOFT_LEFT
                || keyCode == KeyEvent.KEYCODE_SOFT_RIGHT
                || keyCode == KeyEvent.KEYCODE_ENTER
                || keyCode == NoTouchConstant.INIT_KEY_CODE){
            return true;
        }

        return false;
    }

    public boolean getSupportHardKey(){
        return mSupportHardKey;
    }



    /**
     * 取消焦点，将当前焦点移交到statusbar上
     */
    public void cancelFocus(){
        try {
            ContentValues values = new ContentValues();
            values.put(COLUMN_FOCUS_STATE, SYSTEM_BAR_FOCUS);
            values.put(COLUMN_FOCUS_KEY, mAppLastKeyCode);
            mContext.getContentResolver().update(CONTENT_URI, values, null, null);
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    /**
     * 更新当前的焦点是在APP中还是在statusbar中
     */
    private void updateCurrentFocus(){
        Cursor cursor = null;
        try {
            cursor = mContext.getContentResolver().query(CONTENT_URI, null, null, null ,null);
            if(cursor != null && cursor.moveToFirst()){
                mCurrentFocus = cursor.getInt(cursor.getColumnIndex(COLUMN_FOCUS_STATE));
                mNavigationBarKeyCode = cursor.getInt(cursor.getColumnIndex(COLUMN_FOCUS_KEY));
                NoTouchLog.d("mCurrentFocus = " + mCurrentFocus);
            }
        } catch (Exception e) {
            e.printStackTrace();
        }finally {
            if(cursor != null){
                cursor.close();
            }
        }

    }

    /**
     * 设置当前APP需要焦点，可以进行触控操作
     * @param needFocus
     */
    private void sendCurrentNeedFocus(boolean needFocus){
        try {
            NoTouchLog.d("sendCurrentNeedFocus");
            ContentValues values = new ContentValues();
            values.put(COLUMN_FOCUS_NAME, mContext.getPackageName() + ":" + Process.myPid());
            values.put(COLUMN_FOCUS_NEED, needFocus);
            values.put(COLUMN_FOCUS_TEMP, false);
            mContext.getContentResolver().update(NEED_FOCUS_URL, values, null, null);
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    /**
     * 设置当前APP获取焦点
     */
    private void sendCurrentHasFocus(){
        try {
            ContentValues values = new ContentValues();
            values.put(COLUMN_FOCUS_STATE, APP_FOCUS);
            values.put(COLUMN_FOCUS_KEY, mAppLastKeyCode);
            mContext.getContentResolver().update(CONTENT_URI, values, null, new String[]{COLUMN_FOCUS_STATE});
            mCurrentFocus = APP_FOCUS;
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    private class OperationHandler extends Handler{
        public OperationHandler(Looper looper){
            super(looper);
        }

        @Override
        public void handleMessage(Message msg) {
            switch (msg.what){
                case MESSAGE_KEY_DOWN:
                    if(mCurrentFocus == SYSTEM_BAR_FOCUS){
                        return;
                    }

                    if(mNoTouchOperation != null){
                        mNoTouchOperation.operation(msg.arg1, (KeyEvent) msg.obj);
                    }

                    break;
                case MESSAGE_UPDATE_FOCUS:{
                    updateCurrentFocus();
                    boolean isCached = (boolean) msg.obj;
                    if(isCached && mCurrentFocus == APP_FOCUS && mSupportHardKey){
                        mNoTouchOperation.restoreLastCoordinate();
                    }else if(mCurrentFocus == APP_FOCUS){
                        onKeyDown(NoTouchConstant.INIT_KEY_CODE, null);
                    }
                }
                    break;
                default:
                    break;
            }
        }
    }

    private class NoTouchActivityOperation implements ActivityOperationListener{

        @Override
        public ActivityOperationBean activityOperation(OperationBean operationBean) {
            ActivityOperationBean result = new ActivityOperationBean();
            NoTouchLog.d("activityOperation: mNoTouchListener=" + mNoTouchListener + ", operationBean=" + operationBean);
            if(mNoTouchListener != null && operationBean != null){
                switch (operationBean.getOperationFlag()){
                    case NoTouchConstant.OperationFlag.OPERATION_NONE:
                        result.setOperationResult(SUCCESS);
                        break;
                    case NoTouchConstant.OperationFlag.OPERATION_NO_FOCUS:
                        cancelFocus();
                        result = mNoTouchListener.onSelectNone();
                        break;
                    case NoTouchConstant.OperationFlag.OPERATION_SELECTED:
                        result = mNoTouchListener.onSelectedChange(operationBean.getCoordinateBean());
                        break;
                    case NoTouchConstant.OperationFlag.OPERATION_ENTER:
                        result = mNoTouchListener.onSelectedEnter(operationBean.getCoordinateBean());
                        break;
                    default:
                        break;
                }
            }

            return result;
        }
    }

    public void setRecoverCoordinateBean(String className, CoordinateBean coordinateBean){
        if(mNoTouchOperationCache != null && className != null){
            NoTouchOperation noTouchOperation = mNoTouchOperationCache.get(className);
            if(noTouchOperation != null){
                noTouchOperation.setLastCoordinate(coordinateBean);
            }
        }
    }

    public interface ActivityOperationListener{
        ActivityOperationBean activityOperation(OperationBean operationBean);
    }

}
