package com.cneeds.notouch;

import android.content.Context;
import android.view.KeyEvent;

import com.cneeds.notouch.bean.CoordinateBean;

/**
 * 非触控管理类，提供外部调用的接口
 * @author zhany
 */
public class NoTouchManager {

    protected NoTouchManager(){

    }

    private static final class NoTouchManagerHolder{
        private static final NoTouchManager INSTANCE = new NoTouchManager();
    }

    public static NoTouchManager getInstance(){
        return NoTouchManagerHolder.INSTANCE;
    }

    private NoTouchFocusController mNoTouchFocusController;


    /**
     * 初始化
     * @param context 建议传递Application的context
     */
    public void init(Context context){
        mNoTouchFocusController = new NoTouchFocusController(context);
    }

    /**
     * 注册非触控监听
     * @param listener
     * @param noTouchParam
     */
    public void registerNoTouchListener(NoTouchListener listener, NoTouchParam noTouchParam){
        mNoTouchFocusController.registerNoTouchListener(listener, noTouchParam);
    }

    /**
     * 在View销毁，Activity不在前台的情况下需要移除非触控监听。
     * @param listener
     */
    public void unRegisterNoTouchListener(NoTouchListener listener){
        mNoTouchFocusController.unRegisterNoTouchListener(listener);
    }

    /**
     * 设置临时的注册，主要为了一些特殊页面，比如当前为A 通过这个注册后会变成B，
     * 取消注册的时候会恢复A
     * @param packageName
     */
    public void registerNoTouchTempListener(String packageName){
        mNoTouchFocusController.registerNoTouchTempListener(packageName);
    }

    public void unRegisterNoTouchTempListener(String packageName){
        mNoTouchFocusController.unRegisterNoTouchTempListener(packageName);
    }

    /**
     * View或者Activity收到系统onkeyDown的调用后调用此方法。
     * @param keyCode
     * @param keyEvent
     */
    public void onKeyDown(int keyCode, KeyEvent keyEvent){
        mNoTouchFocusController.onKeyDown(keyCode, keyEvent);
    }


    /**
     * 设置恢复的位置
     * @param className
     * @param coordinateBean
     */
    public void setRecoverCoordinateBean(String className, CoordinateBean coordinateBean){
        mNoTouchFocusController.setRecoverCoordinateBean(className , coordinateBean);
    }

    /**
     * 获取是否支持hardkey
     * @return
     */
    public boolean getSupportHardKey(){
        return mNoTouchFocusController.getSupportHardKey();
    }


    public boolean checkUseKeyCode(int keyCode){
        return mNoTouchFocusController.checkUseKeyCode(keyCode);
    }

    /****
     * 以下为页面自己处理点击事件的接口
     * **************************************************
     */

    public void registerSelfListener(NoTouchSelfListener selfListener){
        mNoTouchFocusController.registerSelfListener(selfListener);
    }

    public void unRegisterSelfListener(NoTouchSelfListener selfListener){
        mNoTouchFocusController.unRegisterSelfListener(selfListener);
    }

    public void cancelFocus(){
        mNoTouchFocusController.cancelFocus();
    }

    /**
     * *******************************************************************8
     */

}
