package com.cneeds.rvhelpers;

import android.content.Context;
import android.graphics.PointF;
import android.support.v7.widget.GridLayoutManager;
import android.support.v7.widget.LinearSmoothScroller;
import android.support.v7.widget.RecyclerView;
import android.util.AttributeSet;

import com.cneeds.rvhelpers.adapter.GridRecyclerAdapter;
import com.cneeds.rvhelpers.entity.IHeader;

import java.util.List;

/**
 * 参考博客：https://blog.csdn.net/iblade/article/details/90449089
 * author: maiqh
 */
public class CenterGridLayoutManager extends GridLayoutManager {

    public CenterGridLayoutManager(Context context, int spanCount, GridRecyclerAdapter adapter) {
        super(context, spanCount);
        init(adapter);

    }

    public CenterGridLayoutManager(Context context, AttributeSet attrs, int defStyleAttr, int defStyleRes,  GridRecyclerAdapter adapter) {
        super(context, attrs, defStyleAttr, defStyleRes);
        init(adapter);
    }

    private void init(final GridRecyclerAdapter adapter) {
        setSpanSizeLookup(new SpanSizeLookup() {
            @Override
            public int getSpanSize(int position) {
                if (adapter.getDataList().get(position) instanceof IHeader) {
                    return getSpanCount();
                } else {
                    return 1;
                }
            }
        });
    }


    @Override
    public void smoothScrollToPosition(RecyclerView recyclerView, RecyclerView.State state, int position) {
        RecyclerView.SmoothScroller smoothScroller = new CenterSmoothScroller(recyclerView.getContext());
        smoothScroller.setTargetPosition(position);
        startSmoothScroll(smoothScroller);
    }

    private static class CenterSmoothScroller extends LinearSmoothScroller {

        CenterSmoothScroller(Context context) {
            super(context);
        }

        @Override
        public int calculateDtToFit(int viewStart, int viewEnd, int boxStart, int boxEnd, int snapPreference) {
            // 滑动到中间
            return (boxStart + (boxEnd - boxStart) / 2) - (viewStart + (viewEnd - viewStart) / 2);
        }

//        @Override
//        protected float calculateSpeedPerPixel(DisplayMetrics displayMetrics) {
//            // 减低滑动速度，默认是25f
//            return 50f / displayMetrics.densityDpi;
//        }

        @Override
        public PointF computeScrollVectorForPosition(int targetPosition) {
            return null;
        }
    }

}
